<?php

require_once 'system_config.php';

class Cache
{
    private $main_cache_url;

    private $icon_prefix;
    private $tv_channel_ids;
    private $tv_channel_by_id;
    private $tv_group_ids;
    private $tv_group_by_id;
    private $tv_section_by_id;

    private $group_ndx_by_id;

    private $pf_playback_available;

    public function __construct()
    {
        $this->main_cache_url = MainHelper::$main_cache_url;
    }

    public function clear_all()
    {
        unset($this->icon_prefix);
        unset($this->tv_channel_ids);
        unset($this->tv_channel_by_id);
        unset($this->tv_group_ids);
        unset($this->tv_group_by_id);
        unset($this->tv_section_by_id);
        unset($this->group_ndx_by_id);
    }

    /**
     * @return false if timestamp not changes and cache loading shoud be
     * skipped.
     */
    private function do_check_timestamp_from_ext_api()
    {
        $old_ts = is_file(MainHelper::$main_ts_path) ?
            file_get_contents(MainHelper::$main_ts_path) : '';

        try
        {
            $doc = HD::http_get_document(MainHelper::$main_ts_url);
        }
        catch (Exception $ex)
        {
            $doc = '';
        }

        if (!$doc)
        {
            hd_print("cache W: failed to fetch timestamp");
            return true;
        }

        $ts = trim($doc);
        if ($ts == $old_ts)
        {
            hd_print("cache D: timestamp not changed ($ts)");
            return false;
        }

        hd_print("cache D: timestamp changed '$old_ts' -> '$ts'");
        file_put_contents(MainHelper::$update_ts_path, $ts);

        return true;
    }

    private function do_load_main_cache_from_ext_api()
    {
        clearstatcache();

        if (!$this->do_check_timestamp_from_ext_api())
            return true;

        try
        {
            $doc = HD::http_get_document(MainHelper::$main_cache_url);

            $zip_path = MainHelper::$zip_path;
            if (!HD::write_to_file_using_tmp($zip_path, $doc, "cache"))
                return false;

            // TODO: diff changes?

            HD::safe_unlink(MainHelper::$main_cache_path);

            $fnames = scandir(MainHelper::$base_data_path);
            if ($fnames)
            {
                foreach ($fnames as $fname)
                {
                    if (0 !== strpos($fname, "tv_cache"))
                        continue;

                    $path = MainHelper::$base_data_path . "/$fname";
                    hd_print("cache I: deleting $path");
                    unlink($path);
                }
            }

            $dir = MainHelper::$base_data_path;
            $name = MainHelper::$zip_name;
            $cmd = "cd '$dir' && unzip '$name'";
            hd_print("Cache I: running $cmd");
            system($cmd, $ret);
            unlink($zip_path);
            if ($ret != 0)
            {
                hd_print("Cache E: unzip failed");
                return false;
            }
            if (!is_file(MainHelper::$update_cache_path))
            {
                hd_print("Cache E: cache file not found unexpectedly");
                return false;
            }

            $from = MainHelper::$update_cache_path;
            $to = MainHelper::$main_cache_path;
            if (!rename($from, $to))
            {
                hd_print("Cache E: rename($from, $to) failed");
                return false;
            }

            $from = MainHelper::$update_ts_path;
            $to = MainHelper::$main_ts_path;
            if (!rename($from, $to))
                hd_print("Cache W: rename($from, $to) failed");

            EpfsHandler::write_data_ts(EpfsHandler::DATA_TS);
            return true;
        }
        catch (Exception $e)
        {
            hd_print("Exception loading cache: " . print_r($e,1));
            return false;
        }
    }

    public function tv_cache_exist($section_id)
    {
        $path = MainHelper::tv_cache_path($section_id);
        return is_file($path);
    }

    public function write_tv_cache($section_id, $obj)
    {
        $doc = json_encode($obj);
        $path = MainHelper::tv_cache_path($section_id);

        $id = MainHelper::tv_cache_id($section_id);
        return HD::write_to_file_using_tmp(
            $path, $doc, "tv_cache[$id]");
    }

    private $cmp_token = '';
    private $cmp_lang_code = '';
    private $cmp_country_code = '';

    private function cmp_group_ids_by_country_and_lang($id1, $id2)
    {
        $has_g1 = isset($this->tv_group_by_id[$id1]);
        $has_g2 = isset($this->tv_group_by_id[$id2]);

        if (!$has_g1 && !$has_g2)
            return 0;

        if ($has_g1 != $has_g2)
            return $has_g1 ? -1 : 1;

        $g1 = $this->tv_group_by_id[$id1];
        $g2 = $this->tv_group_by_id[$id2];

        $country_code = $this->cmp_country_code;
        if ($country_code != "")
        {
            $has_country1 = isset($g1['country']) && ($g1['country'] == $country_code);
            $has_country2 = isset($g2['country']) && ($g2['country'] == $country_code);
            if ($has_country1 != $has_country2)
                return $has_country1 ? -1 : 1;
        }

        $plc1 = $g1['preferred_lang_codes'];
        $plc2 = $g2['preferred_lang_codes'];

        $section_lang_code = $this->cmp_lang_code;
        $has_lang1 = in_array($section_lang_code, $plc1);
        $has_lang2 = in_array($section_lang_code, $plc2);

        if ($has_lang1 != $has_lang2)
            return $has_lang1 ? -1 : 1;

        $section_lang_code_weak = $section_lang_code . '_w';
        $has_lang_weak1 = in_array($section_lang_code_weak, $plc1);
        $has_lang_weak2 = in_array($section_lang_code_weak, $plc2);

        if ($has_lang_weak1 != $has_lang_weak2)
            return $has_lang_weak1 ? -1 : 1;

        return 0;
    }

    private static $priority_by_suffix = array(
        "FHD" => 1,
        "HD" => 2,
        "HEVC" => 3,
        "SD" => 4,
        "" => 5,
    );

    public function cmp_group_ids_by_name($id1, $id2)
    {
        $has_g1 = isset($this->tv_group_by_id[$id1]);
        $has_g2 = isset($this->tv_group_by_id[$id2]);

        if (!$has_g1 && !$has_g2)
            return 0;

        if ($has_g1 != $has_g2)
            return $has_g1 ? -1 : 1;

        $g1 = $this->tv_group_by_id[$id1];
        $g2 = $this->tv_group_by_id[$id2];

        $name1 = $g1["name"];
        $name2 = $g2["name"];
        $matches = array();

        $base_name1 = "";
        $suffix1 = "";
        if (preg_match('/(.*) (HD|FHD|HEVC|SD) \(.*\)$/', $name1, $matches))
        {
            $base_name1 = $matches[1];
            $suffix1 = $matches[2];
        }
        else if (preg_match('/(.*) \(.*\)$/', $name1, $matches))
        {
            $base_name1 = $matches[1];
        }

        $base_name2 = "";
        $suffix2 = "";
        if (preg_match('/(.*) (HD|FHD|HEVC|SD) \(.*\)$/', $name2, $matches))
        {
            $base_name2 = $matches[1];
            $suffix2 = $matches[2];
        }
        else if (preg_match('/(.*) \(.*\)$/', $name2, $matches))
        {
            $base_name2 = $matches[1];
        }

        if ($base_name1 != $base_name2 || $base_name1 == "")
            return 0;

        if ($suffix1 != $suffix2)
        {
            $p1 = self::$priority_by_suffix[$suffix1];
            $p2 = self::$priority_by_suffix[$suffix2];
            return $p1 - $p2;
        }

        return 0;
    }

    private function cmp_group_ids($id1, $id2)
    {
        $token = $this->cmp_token;
        if ($token != "all")
        {
            $cmp = $this->cmp_group_ids_by_country_and_lang($id1, $id2);
            if ($cmp != 0)
                return $cmp;
        }

        $cmp = $this->cmp_group_ids_by_name($id1, $id2);
        if ($cmp != 0)
            return $cmp;

        $has_id1 = isset($this->group_ndx_by_id[$id1]);
        $has_id2 = isset($this->group_ndx_by_id[$id2]);

        if (!$has_id1 && !$has_id2)
            return 0;
        else if (!$has_id2)
            return -1;
        else if (!$has_id1)
            return 1;

        if (!$has_id1 && !$has_id2)
            return $id1 - $id2;

        if ($has_id1 != $has_id2)
            return $has_id1 ? -1 : 1;

        $ndx1 = $this->group_ndx_by_id[$id1];
        $ndx2 = $this->group_ndx_by_id[$id2];

        return $ndx1 - $ndx2;
    }

    private function sort_categories(&$group_ids, $token, $lang_code, $country_code)
    {
        $this->cmp_token = $token;
        $this->cmp_lang_code = $lang_code;
        $this->cmp_country_code = $country_code;
        usort($group_ids, array($this, "cmp_group_ids"));
    }

    private $country_code_uc = '';
    private $lang_code = '';

    // private $tokens_default = array('all', 'sport', 'music', 'xxx', 'usa_en', 'uk_en', '24_7_en');
    private $tokens_default = array('all', 'sport', 'music', 'xxx');
    private $tokens_before_country = array('all');

    public function get_def_section_ids()
    {
        $ids = array();
        foreach ($this->tv_section_by_id as $id => $s)
            $ids[] = $id;

        return $ids;
    }

    public function get_main_section_ids()
    {
        $ids = array();
        foreach ($this->tv_section_by_id as $id => $s)
        {
            $has_country = ($this->country_code_uc != "") && isset($s['country_codes']) &&
                in_array($this->country_code_uc, $s['country_codes']);

            $has_lang = ($this->lang_code != "") && isset($s['lang_codes']) &&
                in_array($this->lang_code, $s['lang_codes']);

            $token = isset($s['token']) ? $s['token'] : '';

            if ($has_country || $has_lang ||
                in_array($token, $this->tokens_default))
            {
                if ($token != 'all')
                    $ids[] = $id;
            }
        }
        return $ids;
    }

    public function get_other_section_ids($main_section_ids)
    {
        $idset = array();
        foreach ($main_section_ids as $id)
            $idset[$id] = 1;

        $ids = array();
        foreach ($this->tv_section_by_id as $id => $s)
        {
            if (isset($idset[$id]))
                continue;

            $token = isset($s['token']) ? $s['token'] : '';
            if ($token != 'all')
                $ids[] = $id;
        }
        return $ids;
    }

    private function cmp_sections($s1, $s2)
    {
        $token1 = isset($s1['token']) ? $s1['token'] : '';
        $token2 = isset($s2['token']) ? $s2['token'] : '';

        $token_before_country1 = in_array($token1, $this->tokens_before_country);
        $token_before_country2 = in_array($token2, $this->tokens_before_country);

        if ($token_before_country1 != $token_before_country2)
            return $token_before_country1 ? -1 : 1;

        if ($this->country_code_uc != "")
        {
            $has_country1 = isset($s1['country_codes']) && in_array($this->country_code_uc, $s1['country_codes']);
            $has_country2 = isset($s2['country_codes']) && in_array($this->country_code_uc, $s2['country_codes']);
            if ($has_country1 != $has_country2)
                return $has_country1 ? -1 : 1;
        }

        $token_is_default1 = in_array($token1, $this->tokens_default);
        $token_is_default2 = in_array($token2, $this->tokens_default);

        if ($token_is_default1 != $token_is_default2)
            return $token_is_default1 ? -1 : 1;

        if ($token_is_default1 && $token1 != $token2)
        {
            $index1 = array_search($token1, $this->tokens_default);
            $index2 = array_search($token2, $this->tokens_default);
            return $index1 - $index2;
        }

        if ($this->lang_code != "")
        {
            $has_lang1 = isset($s1['lang_codes']) && in_array($this->lang_code, $s1['lang_codes']);
            $has_lang2 = isset($s2['lang_codes']) && in_array($this->lang_code, $s2['lang_codes']);
            if ($has_lang1 != $has_lang2)
                return $has_lang1 ? -1 : 1;
        }

        $has_name1 = isset($s1['name']);
        $has_name2 = isset($s2['name']);
        if ($has_name1 != $has_name2)
            return $has_name1 ? -1 : 1;

        return strcasecmp($s1['name'], $s2['name']);
    }

    private static function section_ru_is_present($country_code, $lang_code)
    {
        $country_in_cis = CountryHelper::country_in_cis($country_code);
        $country_in_eu_or_us = CountryHelper::country_in_eu($country_code) || ($country_code == 'US');

        $lang_is_ru = ($lang_code == 'ru');

        return $country_in_cis || ($lang_is_ru && !$country_in_eu_or_us);
    }

    private function parse_main_cache($doc)
    {
        $obj = json_decode($doc, true);
        if (!$obj)
        {
            hd_print("cache E: failed to parse cache file $path");
            return false;
        }

        $cfg = SystemConfig::get_instance();
        $cfg->ensure_loaded();
        $this->country_code_uc = $cfg->country_code_uc;
        $this->lang_code = $cfg->lang_code;

        $this->icon_prefix = HD::get($obj, 'icon_prefix', '');
        $this->tv_section_by_id = $obj['sections'];
        $this->tv_group_by_id = $obj['categories'];
        $this->tv_group_ids = $obj['category_ids'];
        $this->tv_channel_by_id = $obj['channels'];
        $this->tv_channel_ids = $obj['channel_ids'];

        foreach ($this->tv_group_ids as $ndx => $id)
            $this->group_ndx_by_id[$id] = $ndx;

        if (!self::section_ru_is_present($this->country_code_uc, $this->lang_code))
            unset($this->tv_section_by_id['russia']);

        uasort($this->tv_section_by_id, array($this, "cmp_sections"));

        foreach ($this->tv_section_by_id as &$section)
        {
            $this->sort_categories(
                $section['categories'],
                HD::get($section, 'token', ''),
                HD::get($section, 'lang_code', ''),
                $this->country_code_uc);
        }
        unset($section);

        return true;
    }

    private function load_main_cache_from_ext_api($force_load)
    {
        $path = MainHelper::$main_cache_path;

        $ts = microtime(true);

        if (!$force_load)
        {
            if (!is_file($path))
            {
                hd_print("Cache file not exist");
                $force_load = true;
            }
/*
            // TODO: revise. Currently commented-out, hoping that timer
            // action will force update in more appropriate time.
            else
            {
                $tdiff = time() - filemtime($path);
                if ($tdiff > 14400)
                {
                    hd_print("Cache file out-of-date (+${tdiff}s)");
                    $force_load = true;
                }
            }
*/
        }

        if ($force_load)
        {
            if (!$this->do_load_main_cache_from_ext_api())
            {
                if (!is_file($path))
                    return false;
            }
        }

        $ts0 = microtime(true);

        hd_print("cache D: reading file");
        $doc = file_get_contents($path);
        if (!$doc)
        {
            hd_print("cache E: failed to read file $path");
            return false;
        }

        $ts1 = microtime(true);

        $res = $this->parse_main_cache($doc);

        $ts2 = microtime(true);
        hd_print(sprintf("cache D: load %.2fs, %.2fs, %.2fs",
                $ts0 - $ts, $ts1 - $ts0, $ts2 - $ts1));

        return $res;
    }

    public function ensure_cache_loaded()
    {
        MainHelper::initialize();

        $force_load = ($this->main_cache_url != MainHelper::$main_cache_url);
        if (!$force_load)
        {
            if (isset($this->tv_channel_by_id) &&
                isset($this->tv_group_by_id) &&
                isset($this->tv_section_by_id) &&
                isset($this->tv_channel_ids) &&
                isset($this->tv_group_ids))
            {
                return true;
            }
        }

        $this->tv_channel_ids = array();
        $this->tv_channel_by_id = array();
        $this->tv_group_ids = array();
        $this->tv_group_by_id = array();
        $this->tv_section_by_id = array();
        $this->group_ndx_by_id = array();

        $result = $this->load_main_cache_from_ext_api($force_load);

        $this->main_cache_url = MainHelper::$main_cache_url;

        if ($result)
        {
            hd_print("DEBUG number of channels: " . count($this->tv_channel_by_id));
            hd_print("DEBUG number of groups: " . count($this->tv_group_by_id));
            hd_print("DEBUG number of sections: " . count($this->tv_section_by_id));
        }

        return $result;
    }

    public function get_channel_ids()
    {
        return $this->tv_channel_ids;
    }

    public function get_channels_count()
    {
        return count($this->tv_channel_ids);
    }

    public function get_channel_by_id_map()
    {
        return $this->tv_channel_by_id;
    }

    public function get_group_by_id_map()
    {
        return $this->tv_group_by_id;
    }

    public function get_section_by_id_map()
    {
        return $this->tv_section_by_id;
    }

    public function get_channel_by_id($id)
    {
        return $this->tv_channel_by_id[$id];
    }

    public function get_channel_by_ndx($ndx)
    {
        $id = $this->tv_channel_ids[$ndx];
        return $this->tv_channel_by_id[$id];
    }

    public function get_group_ids()
    {
        return $this->tv_group_ids;
    }

    public function get_group_by_id($id)
    {
        return $this->tv_group_by_id[$id];
    }

    public function get_section_by_id($id)
    {
        return $this->tv_section_by_id[$id];
    }

    public function get_channel_panel_icon_url($c)
    {
        $stream_id = $c['stream_id'];
        return MainHelper::BASE_EXT_API_URL . "channel_panel_icons/${stream_id}.png";
    }

    public function get_channel_icon_url($c)
    {
        $stream_id = $c['stream_id'];
        return MainHelper::BASE_EXT_API_URL . "channel_icons/${stream_id}.png";
    }

    public function set_pf_playback_available($pf_playback_available)
    {
        $this->pf_playback_available = $pf_playback_available;

        $path = MainHelper::get_pf_playback_available_filepath();
        $content = $this->pf_playback_available . "\n";
        file_put_contents($path, $content);
    }

    public function get_pf_playback_available()
    {
        if ($this->pf_playback_available === null)
        {
            $path = MainHelper::get_pf_playback_available_filepath();
            $content = is_file($path) ? trim(file_get_contents($path)) : "";
            $this->pf_playback_available = ($content == "1" || $content == "true");
        }

        return $this->pf_playback_available;
    }
}

?>
