<?php

class EditListDialog implements UserInputHandler
{
    private $title;
    private $width;

    private $all_by_id;
    private $initial_checked_idset;
    private $checked_idset;
    private $group_by_id;
    private $changed;

    private $apply_action;

    private $sel_ndx = 0;

    public function __construct()
    {
        UserInputHandlerRegistry::get_instance()->
            register_handler($this);
    }

    public function get_handler_id()
    {
        return 'edit_list_dialog';
    }

    public function custom_action($control_id, $params=null)
    {
        return UserInputHandlerRegistry::create_action(
            $this, $control_id, $params);
    }

    public function show($title, $width,
        $all_by_id, $checked_idset, $group_by_id, $apply_action)
    {
        $this->title = $title;
        $this->width = $width;
        $this->all_by_id = $all_by_id;
        $this->initial_checked_idset = $checked_idset;
        $this->checked_idset = $checked_idset;
        $this->group_by_id = $group_by_id;
        $this->apply_action = $apply_action;
        $this->changed = false;

        if (count($this->all_by_id) == 0)
        {
            $this->cleanup();
            return null;
        }

        $DIALOG_H = 870;
        $defs = $this->get_defs();
        $actions = $this->get_actions();
        return ActionFactory::show_dialog($this->title,
            $defs, false, $this->width,
            array('max_height' => $DIALOG_H,
                'actions' => $actions,
                'params' => array('page_size' => 11)));
    }

    private function cleanup()
    {
        unset($this->title);
        unset($this->all_by_id);
        unset($this->checked_idset);
        unset($this->initial_checked_idset);
        unset($this->apply_action);
    }

    private function add_text(&$items, $text, $dy=0, $color=-1) {
        $arr = array('type' => 'text', 'text' => $text);
        if ($dy != 0)
            $arr['dy'] = $dy;
        if ($color != -1)
            $arr['color'] = $color;
        $items[] = $arr;
    }

    private function add_icon(&$items, $icon_url, $w, $h) {
        $items[] = array('type' => 'icon', 'width' => $w,
            'height' => $h, 'url' => $icon_url);
    }

    private function add_gap(&$items, $gap) {
        $items[] = array('type' => 'gap', 'width' => $gap);
    }

    private function add_button(&$defs, $name, $caption, $icon_url, $checked,
        $add_params, $gui_params)
    {
        $ICON_W = 48;
        $ICON_H = 48;
        $ICON_RIGHT = 20;
        $BUTTON_W = $this->width;
        $text_dy = ($ICON_H - 48) / 2;

        $items = array();
        if ($checked >= 2) {
            $this->add_gap($items, $ICON_RIGHT);
            if ($checked == 3) {
                $this->add_text($items, '[x]', $text_dy);
            } else if ($checked == 4) {
                $this->add_text($items, '[x]', $text_dy, 40);
            } else {
                $this->add_text($items, '[', $text_dy);
                $this->add_gap($items, 18);
                $this->add_text($items, ']', $text_dy);
            }
        } else if ($checked >= 0) {
            if ($checked)
                $this->add_text($items, '(x)', $text_dy);
            else
            {
                $this->add_text($items, '(', $text_dy);
                $this->add_gap($items, 18);
                $this->add_text($items, ')', $text_dy);
            }
        }
        $this->add_gap($items, $ICON_RIGHT);
        $this->add_icon($items, $icon_url, $ICON_W, $ICON_H);
        $this->add_gap($items, $ICON_RIGHT);
        $this->add_text($items, $caption, $text_dy);

        $caption = json_encode(array('items' => $items));
        ControlFactory::add_button($defs, $this, $add_params,
            $name, null, $caption, $BUTTON_W, $gui_params);
    }

    private function get_defs()
    {
        $defs = array();

        $ICON_W = 48;
        $ICON_H = 48;
        $ICON_RIGHT = 10;
        $text_dy = ($ICON_H - 48) / 2;

        $left_aligned_params = array(
            'button_caption_centered' => 'no',
            'button_style' => 'smart_label',
            'button_height' => ($ICON_H - 4),
        );

        $add_params = null;

        $this->add_button($defs, 'apply', T::g('button_close'),
            'gui_skin://button_icons/apply.aai', -1,
            $add_params, $left_aligned_params);

        foreach ($this->group_by_id as $gid => $g)
        {
            $checked = $this->get_group_checked($gid);
            $this->add_button($defs, "group:$gid",
                $g->caption, null, $checked,
                $add_params, $left_aligned_params);
        }

        foreach ($this->all_by_id as $id => $obj)
        {
            $checked = isset($this->checked_idset[$id]) ? 1 : 0;
            $this->add_button($defs, "item:$id",
                $obj->caption, $obj->icon_url, $checked,
                $add_params, $left_aligned_params);
        }

        return $defs;
    }

    private function get_group_checked($gid)
    {
        $g = $this->group_by_id[$gid];
        $res = -1;
        foreach ($g->ids as $id) {
            $checked = isset($this->checked_idset[$id]) ? 1 : 0;
            if ($res == -1)
                $res = $checked ? 1 : 0;
            else if ($res != $checked)
                return 4;
        }
        return $res + 2;
    }

    private function get_actions()
    {
        $apply_action = $this->custom_action('apply');

        return array(
            GUI_EVENT_KEY_D_BLUE => $apply_action,
            GUI_EVENT_KEY_RETURN => $apply_action,
        );
    }

    private function validate_changes($user_input)
    {
        if (!$this->checked_idset)
            return ActionFactory::show_error(false, T::T("empty_list_not_allowed"));
        return null;
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        hd_print('EditListDialog: handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        $control_id = HD::get($user_input, 'control_id', '');

        if ($control_id == 'cancel')
            return ActionFactory::close_dialog();

        if (0 === strpos($control_id, 'group:'))
        {
            $gid = substr($control_id, strlen('group:'));

            $checked = $this->get_group_checked($gid);
            $need_unset = $checked == 3;

            $g = $this->group_by_id[$gid];
            foreach ($g->ids as $id) {
                if ($need_unset)
                    unset($this->checked_idset[$id]);
                else
                    $this->checked_idset[$id] = 1;
            }

            $this->changed = true;

            return ActionFactory::reset_controls(
                $this->get_defs());
        }

        if (0 === strpos($control_id, 'item:'))
        {
            $id = substr($control_id, strlen('item:'));
            if (isset($this->checked_idset[$id])) {
                hd_print("Edit D: disable $id");
                unset($this->checked_idset[$id]);
            } else {
                hd_print("Edit D: enable $id");
                $this->checked_idset[$id] = 1;
            }

            $this->changed = true;

            return ActionFactory::reset_controls(
                $this->get_defs());
        }

        if ($control_id == 'apply')
        {
            $a = $this->validate_changes($user_input);
            if ($a)
                return $a;

            return ActionFactory::close_dialog_and_run(
                $this->custom_action('do_apply'));
        }

        if ($control_id == 'do_apply')
        {
            if (!$this->changed)
            {
                $this->cleanup();
                return null;
            }

            $checked_str = json_encode($this->checked_idset);
            $initial_checked_str = json_encode($this->initial_checked_idset);
            $action = $this->apply_action;
            ActionFactory::set_action_param($action,
                'checked_idset', $checked_str);
            ActionFactory::set_action_param($action,
                'initial_checked_idset', $initial_checked_str);
            $this->cleanup();
            return $action;
        }

        return null;
    }
}

?>
