<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/abstract_preloaded_regular_screen.php';

///////////////////////////////////////////////////////////////////////////

class GlobalTvTvChannelListScreen extends AbstractPreloadedRegularScreen
    implements UserInputHandler
{
    const ID = 'tv_channel_list';

    public static function get_media_url_str($group_id, $section_id)
    {
        return MediaURL::encode(array(
            'screen_id'  => self::ID,
            'group_id'   => $group_id,
            'section_id' => $section_id,
        ));
    }

    ///////////////////////////////////////////////////////////////////////

    protected $tv;

    ///////////////////////////////////////////////////////////////////////

    public function __construct($tv, $folder_views)
    {
        parent::__construct(self::ID, $folder_views);
        $this->tv = $tv;
        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        $play_action =
            UserInputHandlerRegistry::create_action(
                $this, 'play_tv');

        $actions = array();
        $actions[GUI_EVENT_KEY_ENTER] = $play_action;//ActionFactory::tv_play();
        $actions[GUI_EVENT_KEY_PLAY] = $play_action;//ActionFactory::tv_play();

        if ($this->tv->is_favorites_supported())
        {
            $add_favorite_action =
                UserInputHandlerRegistry::create_action(
                    $this, 'toggle_favorite');
            $add_favorite_action['caption'] = T::g('plugin_default_favorites');

            $popup_menu_action =
                UserInputHandlerRegistry::create_action(
                    $this, 'popup_menu');

            $actions[GUI_EVENT_KEY_D_BLUE] = $add_favorite_action;
            $actions[GUI_EVENT_KEY_POPUP_MENU] = $popup_menu_action;
        }

        $actions[GUI_EVENT_KEY_INFO] =
            UserInputHandlerRegistry::create_action(
                $this, 'info');

        return $actions;
    }

    public function get_handler_id()
    { return self::ID; }

    public function get_sel_item_update_action(&$user_input, &$plugin_cookies)
    {
        $media_url = MediaURL::decode($user_input->selected_media_url);
        $sel_ndx = $user_input->sel_ndx;
        $group_id = $media_url->group_id;
        $section_id = $media_url->section_id;
        $channel_id = $media_url->channel_id;

        $c = $this->tv->get_channel_by_id($channel_id);
        $items[] = $this->get_regular_folder_item($group_id, $section_id,
            $c, $plugin_cookies);
        $range = HD::create_regular_folder_range($items,
            $sel_ndx, $this->tv->get_channels_count($group_id));

        return ActionFactory::update_regular_folder($range, false);
    }

    ///////////////////////////////////////////////////////////////////////

    private function get_regular_folder_item($group_id, $section_id, $c, &$plugin_cookies)
    {
        $img = HD::enable_caching(
            $this->tv->get_channel_panel_icon_url($c));
        $media_url =  MediaURL::encode(array(
            'channel_id' => $c['id'],
            'group_id' => $group_id,
            'section_id' => $section_id,
        ));

        return array
        (
            PluginRegularFolderItem::media_url => $media_url,
            PluginRegularFolderItem::caption => $c['name'],
            PluginRegularFolderItem::view_item_params => array
            (
                ViewItemParams::icon_path => $img,
                ViewItemParams::item_detailed_icon_path => $img,
            ),
            PluginRegularFolderItem::starred =>
                $this->tv->is_favorite_channel_id(
                    $c['id'], $plugin_cookies),
        );
    }

    public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies)
    {
        $this->tv->ensure_channels_loaded();

        $items = array();

        $all_g = $this->tv->get_all_channels_group();
        $fav_g = $this->tv->get_favorites_group();
        if ($media_url->group_id == $fav_g['id']) {
            $channel_ids = $this->tv->get_fav_channel_ids($plugin_cookies);
        } else if ($media_url->group_id == $all_g['id']) {
            $channel_ids = $this->tv->get_channel_ids();
        } else {
            $g = $this->tv->get_group($media_url->group_id);
            $channel_ids = $g['channels'];
        }

        foreach ($channel_ids as $cid)
        {
            $c = $this->tv->get_channel_by_id($cid);
            $tvc = $this->get_regular_folder_item(
                $media_url->group_id, $media_url->section_id, $c, $plugin_cookies);
            $items[] = $tvc;
        }

        return $items;
    }

    public function get_archive(MediaURL $media_url)
    {
        return $this->tv->get_archive($media_url);
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        hd_print('TV channel list: handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        $ch_id = null;
        if (isset($user_input->selected_media_url))
        {
            $media_url = MediaURL::decode($user_input->selected_media_url);
            $ch_id = $media_url->channel_id;
        }

        $control_id = HD::get($user_input, 'control_id');
        if ($control_id == 'info')
        {
            if (!$ch_id)
                return null;

            $channel = $this->tv->get_channel_by_id($ch_id);
            $id = $channel["id"];
            $title = $channel["name"];

            $msg = "TV channel '$title' (id=$id)";
            hd_print($msg);
            return ActionFactory::show_title_dialog($msg);
        }
        else if ($control_id == 'popup_menu')
        {
            if (!$ch_id)
                return null;

            $is_favorite = $this->tv->is_favorite_channel_id(
                $ch_id, $plugin_cookies);
            $caption = $is_favorite ?
                T::g('dune_tv_popup_remove_from_favorites') :
                T::g('dune_tv_popup_add_to_favorites');
            $add_favorite_action =
                UserInputHandlerRegistry::create_action(
                    $this, 'toggle_favorite');
            $menu_items[] = array(
                GuiMenuItemDef::caption => $caption,
                GuiMenuItemDef::action => $add_favorite_action);

            return ActionFactory::show_popup_menu($menu_items);
        }
        else if ($control_id == 'toggle_favorite')
        {
            if (!$ch_id)
                return null;

            $is_favorite = $this->tv->is_favorite_channel_id(
                $ch_id, $plugin_cookies);
            $text = "";

            if ($is_favorite)
            {
                $this->tv->change_tv_favorites(PLUGIN_FAVORITES_OP_REMOVE,
                    $ch_id, $plugin_cookies);
                $text = T::g('tv_channel_removed_from_favorites');
            }
            else
            {
                $this->tv->change_tv_favorites(PLUGIN_FAVORITES_OP_ADD,
                    $ch_id, $plugin_cookies);
                $text = T::g('tv_channel_added_to_favorites');
            }

            $update_action = $this->get_sel_item_update_action($user_input, $plugin_cookies);
            $action = ActionFactory::show_title_dialog($text, $update_action);
            if (!HD::with_rows_api())
                return $action;

            return $this->tv->invalidate_favorites($action);
        }

        $action = MainHelper::get_pf_check_action_if_needed($user_input, "TV channel list", false);
        if ($action != null)
            return $action;

        $a = MainHelper::get_pre_playback_action_if_needed($this, $ch_id, $user_input, $plugin_cookies);
        if ($a !== true)
            return $a;

        $control_id = HD::get($user_input, 'control_id');
        if ($control_id == 'play_tv')
            return ActionFactory::tv_play();

        return null;
    }
}

///////////////////////////////////////////////////////////////////////////
?>
