<?php

class MainHelper
{
    const BASE_EXT_API_URL = "http://s1.dune-club.info/plugins/globaltv/";
    #const BASE_EXT_API_URL = "http://files4.dune-hd.com/xxx/";

    public static $plugin_name = null;

    public static $plugin;
    public static $tv;

    public static $entry_handler;
    public static $cache;

    public static $edit_list_dialog;
    public static $list_helper;

    public static $tv_rows_screen;
    public static $dummy_epf_screen;

    public static $base_tmp_path;
    public static $base_data_path;
    public static $wake_path;
    public static $zip_name;
    public static $zip_path;
    public static $main_cache_path;
    public static $update_cache_path;
    public static $main_ts_path;
    public static $update_ts_path;
    public static $main_cache_url;
    public static $main_ts_url;

    public static $timezone_diff_secs;

    public static function without_internet()
    {
        return self::$plugin ?
            self::$plugin->get_internet_status() == 0 : false;
    }

    public static function initialize(
        $base_tmp_path=null, $base_data_path=null)
    {
        self::$base_tmp_path = $base_tmp_path ? $base_tmp_path :
            DuneSystem::$properties['tmp_dir_path'];
        self::$base_data_path = $base_data_path ? $base_data_path :
            DuneSystem::$properties['data_dir_path'];

        self::$wake_path = self::$base_tmp_path . "/wake_flag";

        self::$zip_name = "main_cache.zip";
        self::$zip_path = self::$base_data_path . "/" . self::$zip_name;

        self::$main_cache_path = self::$base_data_path . "/active_cache.json";
        self::$update_cache_path = self::$base_data_path . "/main_cache.json";
        self::$main_ts_path = self::$base_data_path . "/active_timestamp";
        self::$update_ts_path = self::$base_data_path . "/timestamp";

        self::$main_cache_url = self::BASE_EXT_API_URL . 'cache2.zip';
        self::$main_ts_url = self::BASE_EXT_API_URL . 'cache.timestamp';

        $cache_url = '';
        if (is_file("/config/globaltv_cache_url"))
            $cache_url = trim(file_get_contents("/config/globaltv_cache_url"));
        if ($cache_url)
            self::$main_cache_url = $cache_url;

        $ts_url = '';
        if (is_file("/config/globaltv_ts_url"))
            $ts_url = trim(file_get_contents("/config/globaltv_ts_url"));
        if ($ts_url)
            self::$main_ts_url = $ts_url;

        date_default_timezone_set("UTC");
        $timezone_str = exec("date +%z");
        self::$timezone_diff_secs = self::timezone_str_to_diff_secs($timezone_str);

        if (!$base_tmp_path)
            self::$edit_list_dialog = new EditListDialog();
    }

    public static function get_pf_playback_available_filepath()
    {
        return self::$base_data_path . "/pf_playback_available";
    }

    private static function timezone_str_to_diff_secs($timezone_str)
    {
        $h = intval(substr($timezone_str, 0, 3));
        $m = intval(substr($timezone_str, 3, 2));
        #hd_print("timezone_str_to_diff_secs, timezone_str: '$timezone_str', h: $h, m: $m");
        return ($h * 60 + $m) * 60;
    }

    public static function tv_cache_id($section_id)
    {
        return $section_id == -1 ||
            $section_id == SectionsHelper::FAV_SECTION_ID || 
            $section_id == 'fav' ?
            "all" : $section_id;
    }

    public static function tv_cache_path($section_id)
    {
        $str = self::tv_cache_id($section_id);
        return self::$base_data_path . "/tv_cache_$str.json";
    }

    public static function tv_cache_clear_all()
    {
        $fnames = scandir(self::$base_data_path);
        if (!$fnames)
            return;

        foreach ($fnames as $fname)
        {
            if (preg_match('|^tv_cache_.*\.json$|', $fname))
            {
                $path = self::$base_data_path . "/" . $fname;
                hd_print("I: deleting tv cache file: $path");
                if (!unlink($path))
                    hd_print("W: unlink failed");
            }
        }
    }

    public static function tv_password_path()
    {
        return self::$base_data_path . "/tv_password.txt";
    }

    public static function with_favorites()
    {
        $pcookies = self::$plugin->get_plugin_cookies();
        return self::$tv->favorites_nonempty($pcookies);
    }

    public static function show_error_dialog($error_code, $post_action = null)
    {
        $line1 = T::t("msg_internal_error");
        $line2 = "";
        $line3 = "";
        $add_pay_button = false;
        $use_maintenance_message = true;
        if ($error_code == 201)
        {
            $line1 = T::t("msg_no_access");
            $use_maintenance_message = false;
        }
        else if ($error_code == 202)
        {
            $line1 = T::t("msg_trial_over");
            $line2 = T::t("msg_use_paid_access");
            $use_maintenance_message = false;
        }
        else if ($error_code == 203)
        {
            $line1 = T::t("msg_trial_unavailable");
            $line2 = T::t("msg_try_again_later_or_use_paid_access");
            $use_maintenance_message = false;
            $add_pay_button = true;
        }
        else if ($error_code == 204)
        {
            $line1 = T::t("msg_service_unavailable");
            $line2 = T::t("msg_try_again_later");
        }

        if ($use_maintenance_message)
        {
            $line1 = T::t("msg_service_unavailable");
            $line2 = T::t("msg_repair_work");
            $line3 = T::t("msg_try_again_later");
        }

        $defs = array();

        ControlFactory::add_label($defs, null, $line1);
        if ($line2 != '')
        {
            ControlFactory::add_vgap($defs, -15);
            ControlFactory::add_label($defs, null, $line2);
        }
        if ($line3 != '')
        {
            ControlFactory::add_vgap($defs, -15);
            ControlFactory::add_label($defs, null, $line3);
        }

        if ($add_pay_button)
        {
            $pf_pay_action = self::get_pf_pay_action(null, null);

            ControlFactory::add_custom_button($defs,
                ActionFactory::close_dialog_and_run($pf_pay_action),
                'pay', null, T::key_global('premium_function_button_pay'), 300);
        }

        ControlFactory::add_custom_close_dialog_and_apply_button($defs,
            'ok', 'OK', 300, $post_action);

        return ActionFactory::show_dialog('Global TV', $defs, false, 0);
    }

    public static function get_pf_check_action($post_action, $error_action = null)
    {
        $need_activation = false;
        $ignore_trial = false;

        $trial_status_info = self::$tv->get_trial_status_info();
        if ($trial_status_info['status'] == 'ok')
        {
            $trial_status = $trial_status_info['trial'];
            if ($trial_status->trial_used)
            {
                $expire_ts = $trial_status->expire_ts;
                $ignore_trial = (time() > $expire_ts);
            }
            else
            {
                $need_activation = true;
            }
        }
        else
        {
            hd_print("Error occured, code: " . $url_info['error_code'] . ", error : '" . $url_info['error'] . "'");
        }

        $params = array(
            'need_activation' => $need_activation,
            "ignore_trial" => $ignore_trial,
            "extra_text1" => T::t("msg_dune_extra_needed"),
            "extra_text2" => T::t("msg_globaltv_trial_activate"),
            "extra_text2.only_if_need_activation" => true,
        );

        hd_print("Run pf_check action with params: " . print_r($params, true));

        return ActionFactory::run_builtin('pf_check',
            $params, $post_action, $error_action);
    }

    public static function get_pf_check_action_if_needed($user_input, $debug_name, $use_error_action)
    {
        $pf_checked = isset($user_input->pf_checked) ?
            $user_input->pf_checked : false;
        if ($pf_checked)
            return null;

        hd_print("PF check ($debug_name)");

        $ok_action = ActionFactory::action_from_user_input($user_input,
            array('pf_checked' => true));
        $err_action = ActionFactory::action_from_user_input($user_input,
            array('pf_checked' => true, 'pf_error' => true));
        if (!$use_error_action)
            $err_action = null;

        return self::get_pf_check_action($ok_action, $err_action);
    }

    public static function get_pf_pay_action($post_action, $error_action = null)
    {
        return ActionFactory::run_builtin('pf_pay',
            array(), $post_action, $error_action);
    }

    public static function read_xxx_password()
    {
        $path = "/config/parental_control_password.txt";
        return is_file($path) ? (string) file_get_contents($path) : '';
    }

    public static function remove_from_array_by_value(&$arr, $value)
    {
        $ind = array_search($value, $arr);
        if ($ind === false)
            return false;

        array_splice($arr, $ind, 1);
    }

    public static function get_pre_playback_action_if_needed(UserInputHandler $handler, $ch_id, &$user_input, &$plugin_cookies)
    {
        self::$tv->ensure_channels_loaded();

        $ch = self::$tv->get_channel_by_id($ch_id);
        if (!isset($ch['stream_id']))
            return true;

        $url_info = self::$tv->get_playback_url_info($ch['stream_id'], -1, 0);
        if ($url_info['status'] != 'ok')
        {
            hd_print("Error occured, code: " . $url_info['error_code'] .
                ", error : '" . $url_info['error'] . "'");
            return self::show_error_dialog($url_info['error_code']);
        }

        self::$tv->set_playback_url_for_ch_id($ch_id, $url_info['url']);

        return self::show_xxx_dialog_if_needed($handler, $ch_id, $user_input, $plugin_cookies);
    }

    public static function get_pre_playback_action_for_group_if_needed(UserInputHandler $handler, $group_id, &$user_input, &$plugin_cookies)
    {
        if ($group_id == null)
            return true;

        self::$tv->ensure_channels_loaded();

        $group = self::$tv->get_group($group_id);
        if (!isset($group['channels']) || !isset($group['channels'][0]))
        {
            hd_print("Bad group: " . print_r($group, true));
            return true;
        }

        $ch_id = $group['channels'][0];
        return self::get_pre_playback_action_if_needed($handler, $ch_id, $user_input, $plugin_cookies);
    }

    public static function show_xxx_dialog_temp_access($post_action)
    {
        $defs = array();
        ControlFactory::add_label($defs, null,
            T::t("label_temp_access"));
        ControlFactory::add_label($defs, null,
            T::t("label_change_future"));

        ControlFactory::add_custom_button($defs,
            ActionFactory::close_dialog_and_run($post_action),
            'ok', null, T::key_global('button_ok'), 300);

        return ActionFactory::show_dialog(
            "XXX", $defs, true, 1500);
    }

    public static function show_xxx_dialog_18_content($post_action)
    {
        $defs = array();
        ControlFactory::add_label($defs, null,
            T::t("label_18_content"));

        ControlFactory::add_close_dialog_button($defs,
            T::key_global('button_cancel'), 300);
        ControlFactory::add_custom_button($defs,
            ActionFactory::close_dialog_and_run($post_action),
            'i_am_18', null, T::t("button_18"), 300);

        return ActionFactory::show_dialog(
            "XXX", $defs, true, 1000);
    }

    public static function show_xxx_dialog_code($post_action)
    {
        $action = ActionFactory::change_parental_code($post_action);

        $defs = array();
        ControlFactory::add_label($defs, null,
            T::t('parental_code_recommend_set'));
        ControlFactory::add_label($defs, null,
            T::t('parental_code_can_be_set_in_settings'));
        ControlFactory::add_custom_button($defs,
            ActionFactory::close_dialog_and_run($action),
            'set_parental_code', null, T::t('parental_code_set_now'), 300);

        ControlFactory::add_custom_close_dialog_and_apply_button($defs,
            'skip', T::g('choice_skip'), 300, $post_action);
        return ActionFactory::show_dialog(
            T::g('title_parental_code_not_set'), $defs, true, 1500);
    }

    public static function create_action(UserInputHandler $handler, $control_id, $prev_control_id)
    {
        $params = array('prev_control_id' => $prev_control_id);
        return UserInputHandlerRegistry::create_action($handler, $control_id, $params);
    }

    public static function show_xxx_dialog_if_needed(UserInputHandler $handler, $ch_id, &$user_input, &$plugin_cookies)
    {
        self::$tv->ensure_channels_loaded();
        if (!self::$tv->channel_is_protected($ch_id))
            return true;

        $control_id = HD::get($user_input, 'control_id');
        $prev_control_id = HD::get($user_input, 'prev_control_id');
        if (!$prev_control_id)
            $prev_control_id = $control_id;

        if ($control_id == 'xxx_18_one_dialog_shown')
        {
            $post_action = self::create_action($handler, 'xxx_18_dialogs_shown', $prev_control_id);
            return self::show_xxx_dialog_temp_access($post_action);
        }
        else if ($control_id == 'xxx_18_dialogs_shown')
        {
            $plugin_cookies->xxx_18_dialogs_shown = 1;
        }
        else if ($control_id == 'xxx_code_dialog_shown')
        {
            $code_changed = isset($user_input->code_changed) ?
                $user_input->code_changed : true;
            if (!$code_changed)
                return null;

            $plugin_cookies->xxx_code_dialog_shown = 1;
        }

        $xxx_18_dialogs_shown = isset($plugin_cookies->xxx_18_dialogs_shown) ?
            $plugin_cookies->xxx_18_dialogs_shown : false;

        if (!$xxx_18_dialogs_shown)
        {
            $post_action = self::create_action($handler, 'xxx_18_one_dialog_shown', $prev_control_id);
            return self::show_xxx_dialog_18_content($post_action);
        }

        $xxx_code_dialog_shown = isset($plugin_cookies->xxx_code_dialog_shown) ?
            $plugin_cookies->xxx_code_dialog_shown : false;

        if (!$xxx_code_dialog_shown && self::read_xxx_password() == '')
        {
            $post_action = self::create_action($handler, 'xxx_code_dialog_shown', $prev_control_id);
            return self::show_xxx_dialog_code($post_action);
        }

        $prev_control_id = HD::get($user_input, 'prev_control_id');
        if ($prev_control_id)
            $user_input->control_id = $prev_control_id;

        return true;
    }
}
?>
